// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of cgal-python; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with cgal-python.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id$
// $URL$
//
// Author(s)     : Spiros Evangelatos
//=========================================================================
//
// Pydoc strings for the Polygon cgal-python module.

#ifndef POLYGON_2_DOCS_H
#define POLYGON_2_DOCS_H

namespace Polygon_2_docs
{
	 static const char * polygon = "\
		Implements a polygon in the two-dimensional Euclidean plane ExE.";

	 static const char * insert = "\
		poly.insert(self, i, Point_2 v) -> void\n\
		Inserts the vertex v before the i-th vertex of poly.";

	 static const char * push_back = "\
		poly.push_back(self, Point_2 v) -> void\n\
		poly.push_back(v) <==> poly.insert(poly.size(), p)\
		";

	 static const char * clear = "\
		poly.clear(self) -> void \n\
		Erases all vertices.\
		";
		
	 static const char * reverse_orientation = "\
		poly.reverse_orientation(self) -> void\n\
		Reverses the orientation of the polygon. The vertex poly[0] remains the same.\
		";

	 static const char * erase = "\
		poly.erase(self, i) -> void\n\
		Erases the i-th vertex of poly.\
		";

	 static const char * is_simple = "\
		poly.is_simple(self) -> bool\n\
		Returns whether poly is a simple polygon.\
		";
		
	 static const char * is_convex = "\
		poly.is_convex(self) -> bool\n\
		Returns whether poly is convex.\
		";
		
	 static const char * orientation = "poly.orientation() -> Orientation\nreturns the orientation of poly.";
	 static const char * oriented_side = "poly.oriented_side(q) -> Kernel.Sign\n \
	 	Returns CGAL.Kernel.Sign.SMALLER, CGAL.Kernel.Sign.EQUAL, \n\
		or CGAL.Kernel.Sign.LARGER, depending on where point q is.\n\
		Precondition: 	poly.is_simple().";
	 static const char * bounded_side = "poly.bounded_side(q) -> CGAL.Kernel.Bounded_side\n \
		 Returns Kernel.Bounded_side.ON_BOUNDED_SIDE, Kernel.Bounded_side.ON_BOUNDARY \n\
		 or CGAL.Kernel.Bounded_side.ON_UNBOUNDED_SIDE, depending on whether point q is\n\
		 on the bounded side, the boundary, or the unbounded side of the polygon poly.\n\
		 Precondition:   poly.is_simple().";

	 static const char * bbox = "poly.bbox() -> Bbox_2\nreturns a bounding box containing poly.";
	 static const char * area = "poly.area() -> double\nreturns the signed area of poly.";
	 static const char * left_vertex = "p.left_vertex_index() -> int\n\
	 	Returns the index of the leftmost vertex of the polygon p with the \n\
		smallest y-coordinate.\
		";
	 static const char * right_vertex = "p.right_vertex_index() -> int\n\
	 	Returns the index of the rightmost vertex of the polygon p with the \n\
		largest y-coordinate.\
		";
	 static const char * top_vertex = "p.top_vertex_index() -> int\n\
	 	Returns the index of the topmost vertex of the polygon p with the \n\
		largest x-coordinate.\
		";
	 static const char * bottom_vertex = "p.bottom_vertex_index() -> int\n\
	 	Returns the index of the bottommost vertex of the polygon p with the \n\
		smallest x-coordinate.\
		";
	 static const char * is_counterclockwise_oriented = "\
		poly.is_counterclockwise_oriented(self) -> bool\n\
		Returns whether poly is counterclockwise oriented.\
		";
		
	 static const char * is_clockwise_oriented = "\
		poly.is_clockwise_oriented(self) -> bool\n\
		Returns whether poly is clockwise oriented.\
		";
		
	 static const char * is_collinear_oriented = "\
		poly.is_collinear_oriented(self) -> bool\n\
		Returns whether poly is collinear oriented.\
		";
		
	 static const char * edge = "\
		poly.edge(self, i) -> Segment_2\n\
		Returns a reference to the i-th edge of poly.\
		";

	 static const char * has_on_positive_side = "poly.has_on_positive_side( Point_2 p) -> bool";
	 static const char * has_on_negative_side = "poly.has_on_negative_side( Point_2 p) -> bool";
	 static const char * has_on_boundary = "poly.has_on_boundary( Point_2 p) -> bool";
	 static const char * has_on_bounded_side = "poly.has_on_bounded_side( Point_2 p) -> bool";
	 static const char * has_on_unbounded_side = "poly.has_on_unbounded_side( Point_2 p) -> bool";

	 static const char * append = "\
		poly.append(self, Point_2 p) -> void\n\
		poly.append(p) <==> poly.push_back(p)";

	 static const char * __getitem__ = "\
		poly[i] -> Point_2\n\
		Returns the i-th vertex of poly.";

	 static const char * __setitem__ = "\
		poly[i] = v\n\
		Modifies the i-th vertex of poly.";

	 static const char * __delitem__ = "\
		del poly[i] <==> poly.erase(i)\n";

	 static const char * __len__ = "\
		len(poly) <==> poly.size()";

	 static const char * size = "\
		poly.size(self) -> int\n\
		Returns the number of vertices of the polygon poly.";

	 static const char * is_empty = "\
		poly.is_empty(self) -> bool\n\
		Returns whether poly.size() == 0.";

	 static const char * __repr__ = "poly.__repr__() <==> repr(poly)";
	 static const char * __str__ = "poly.__str__() <==> str(poly)";
	 static const char * vertices = "The sequence of the vertices of the polygon.";
	 static const char * edges = "The sequence of the edges of the polygon.";
	 static const char * vertices_circulator = "The cyclic sequence of the vertices of the polygon.";
	 static const char * edges_circulator = "The cyclic sequence of the edges of the polygon";
}

#endif
