// Helper class to implement python list-like behavior
// adapted from http://wiki.python.org/moin/boost.python/StlContainers
// Author: Spiros Evangelatos

#ifndef PYLIST_HELPER_H
#define PYLIST_HELPER_H

#include <boost/python/errors.hpp>
using namespace boost::python;

int sanitize_py_index(int i, int len)
{
	if ( i < 0 ) i += len;
	if ( i < 0 || i >= len) {
		PyErr_SetString(PyExc_IndexError, "Index out of range");
		throw_error_already_set();
	}
	return i;
}

int check_positive_index(int i, int len)
{
	if ( i < 0 || i >= len) {
		PyErr_SetString(PyExc_IndexError, "Index out of range");
		throw_error_already_set();
	}
	return i;
}

template<class T, class V>
struct pylist_helper
{
	static const V& get(T const & x, int i)
	{
		i = sanitize_py_index(i, x.size());
		return x[i];
	}
	
	static void set(T &x, int i, V const & v)
	{
		i = sanitize_py_index(i, x.size());
		x[i] = v;
	}
	
	static void del(T &x, int i)
	{
		i = sanitize_py_index(i, x.size());
		x.erase(i);
	}
	
	static void add(T &x, V &v)
	{
		x.push_back(v);
	}
	
};

#endif
