#!/usr/bin/python
print "		======================================================"
print "				testing Alpha_shape_3"
print "		======================================================\n"

from sys import *
path.append("../../cgal_package")

from CGAL.Alpha_shapes_3 import *
from CGAL.Triangulations_3 import Delaunay_triangulation_3

from CGAL.Kernel import *
from random import *

def show_alpha_values(AS):
	print "Alpha spectrum"
	for alpha_value in AS.alpha:
		print alpha_value

def test_Alpha_shapes_3():	
	L =[]
	verbose = True
	
	# first a known small case
	# a cube with one corner less and two small pyramids
	# on back and front face
	L.append(Point_3(0.,0.,0.))
	L.append(Point_3(2.,0.,0.))
	L.append(Point_3(0.,2.,0.))
	L.append(Point_3(0.,0.,2.))
	L.append(Point_3(2.,0.,2.))
	L.append(Point_3(0.,2.,2.))
	L.append(Point_3(2.,2.,0))
	L.append(Point_3(1.,-1.5 ,1.))
	L.append(Point_3(1.,3.5 ,1.))
	
	
	a1 = Alpha_shape_3( 0, Alpha_shape_3.Mode.REGULARIZED)
	a1.make_alpha_shape(L)
	assert(a1.number_of_alphas() == 3 )
	
	print "REGULARIZED  mode" 
	print "test_classify_and_iterators" 
	for alpha in a1.alpha:
		a1.set_alpha(alpha)
		print "alpha value ", alpha
			
	if (verbose):
		a1.print_maps()
	if (verbose):
		a1.print_alphas()
		
	print "test filtration " 
	print "filtration not included yet"
	
	a1.set_mode(Alpha_shape_3.Mode.GENERAL)
	print "GENERAL mode" 
	if(verbose):
		a1.print_maps()
	if(verbose):
		a1.print_alphas()
	assert(a1.number_of_alphas() == 9) 
	print "test_classify_and_iterators" 
			
	for alpha_value in a1.alpha:
		a1.set_alpha(alpha_value)
		print "alpha value ", alpha_value

# 	print "test filtration " 
# 	test_filtration(a1,verbose)
# 	
	# alpha values 0
	#              1
	#              1.0625
	#              1.38942
	#              2
	#              2.00694
	#              2.66667
	#              3
	#              4.35417
	
	
	a1.set_mode(Alpha_shape_3.Mode.REGULARIZED)
	print "REGULARIZED mode" 
	print "test number_of_components - find_optimal_alpha "
	if (verbose):
		a1.print_maps()
		a1.print_alphas()
	
		for alpha_it in a1.alpha:
			print "alpha  " ,alpha_it
			print "number of solid componenst ",a1.number_of_solid_components(alpha_it)
	
	# alpha  2.00694  number of solid components 2
	# alpha  3        number of solid component 1
	# alpha  4.35417  number of solid components 1
	
	assert (a1.find_optimal_alpha(1).next() == a1.get_nth_alpha(2))
	assert (a1.find_optimal_alpha(2).next() == a1.get_nth_alpha(1))
	assert (a1.number_of_solid_components( a1.find_optimal_alpha(2).next() ) == 2)
	assert (a1.number_of_solid_components(a1.find_optimal_alpha(1).next() ) == 1)
	assert (a1.get_nth_alpha(2) == a1.alpha_lower_bound(3).next())
	assert (a1.get_nth_alpha(3) == a1.alpha_upper_bound(4).next())
	
# 	print 
	print "test additionnal creators and set mode" 
	dt2 = Delaunay_triangulation_3()
	dt2.insert(L)
	a2 = Alpha_shape_3(dt2, 0,  Alpha_shape_3.Mode.REGULARIZED)
	if(verbose):
		show_alpha_values(a2)
	if(verbose):
		a2.print_maps()
	if(verbose):
		a2.print_alphas()
	assert(a2.number_of_alphas() == 3) 
# 	
	dt3 = Delaunay_triangulation_3()
	dt3.insert(L)
	a3 = Alpha_shape_3(dt3, 0,  Alpha_shape_3.Mode.GENERAL)
	assert(a3.number_of_alphas() == 9) 
	
	dt4 = Delaunay_triangulation_3();dt4.insert(L)
	a4 = Alpha_shape_3(dt4)
	assert(a4.number_of_alphas() == 3) 
	
	a5 = Alpha_shape_3(0,  Alpha_shape_3.Mode.GENERAL)
	a5.make_alpha_shape(L)
	assert(a5.number_of_alphas() == 9) 
	
	a1.set_mode(Alpha_shape_3.Mode.REGULARIZED)
	assert(a1.number_of_alphas() == 3) 
	
	a1.set_mode(Alpha_shape_3.Mode.GENERAL)
	assert(a1.number_of_alphas() == 9) 

	#========================
	exterior_cells = a1.get_alpha_shape_cells(Alpha_shape_3.Classification_type.EXTERIOR)
	interior_cells = a1.get_alpha_shape_cells(Alpha_shape_3.Classification_type.INTERIOR)
	assert(len(interior_cells) + len(exterior_cells) == a1.number_of_finite_cells())
	
	
	exterior_facets = a1.get_alpha_shape_facets(Alpha_shape_3.Classification_type.EXTERIOR)
	singular_facets = a1.get_alpha_shape_facets(Alpha_shape_3.Classification_type.SINGULAR)
	regular_facets = a1.get_alpha_shape_facets(Alpha_shape_3.Classification_type.REGULAR)
	interior_facets = a1.get_alpha_shape_facets(Alpha_shape_3.Classification_type.INTERIOR)
	
	count_exterior_facets = len(exterior_facets)
	count_singular_facets = len(singular_facets)
	count_regular_facets  = len(regular_facets)
	count_interior_facets = len(interior_facets)
	
	exterior_edges = a1.get_alpha_shape_edges(Alpha_shape_3.Classification_type.EXTERIOR)
	singular_edges = a1.get_alpha_shape_edges(Alpha_shape_3.Classification_type.SINGULAR)
	regular_edges = a1.get_alpha_shape_edges(Alpha_shape_3.Classification_type.REGULAR)
	interior_edges = a1.get_alpha_shape_edges(Alpha_shape_3.Classification_type.INTERIOR)
	
	count_exterior_edges = len(exterior_edges)
	count_singular_edges = len(singular_edges)
	count_regular_edges  = len(regular_edges)
	count_interior_edges = len(interior_edges)
	
	
	exterior_vertices = a1.get_alpha_shape_vertices(Alpha_shape_3.Classification_type.EXTERIOR)
	singular_vertices = a1.get_alpha_shape_vertices(Alpha_shape_3.Classification_type.SINGULAR)
	regular_vertices = a1.get_alpha_shape_vertices(Alpha_shape_3.Classification_type.REGULAR)
	interior_vertices = a1.get_alpha_shape_vertices(Alpha_shape_3.Classification_type.INTERIOR)
	
	count_exterior_vertices = len(exterior_vertices)
	count_singular_vertices = len(singular_vertices)
	count_regular_vertices  = len(regular_vertices)
	count_interior_vertices = len(interior_vertices)
	
	
	if (verbose):	
		print "facets "  
		print count_exterior_facets 
		print count_singular_facets  
		print count_regular_facets 
		print count_interior_facets 
		
		print "edges "  
		print count_exterior_edges 
		print count_singular_edges 
		print count_regular_edges 
		print count_interior_edges 
		
		print "vertices "
		print count_exterior_vertices  
		print count_singular_vertices  
		print count_regular_vertices  
		print count_interior_vertices 

	
	if (a1.get_mode() == Alpha_shape_3.Mode.REGULARIZED):
		assert (count_singular_facets == 0)
		assert (count_exterior_facets + count_regular_facets + count_interior_facets == a1.number_of_finite_facets())
	
		assert(count_singular_edges == 0) 
		assert(count_interior_edges + count_regular_edges + count_exterior_edges == a1.number_of_finite_edges())
		
		assert(count_singular_vertices == 0)
		assert(  count_interior_vertices + count_regular_vertices + count_singular_vertices + count_exterior_vertices == a1.number_of_vertices())
	
# 	
	if( a1.get_mode() == Alpha_shape_3.Mode.GENERAL):
		assert (  count_exterior_facets 
			+ count_singular_facets
			+ count_regular_facets 
			+ count_interior_facets == a1.number_of_finite_facets())
	
		assert(    count_interior_edges
			+ count_regular_edges
			+ count_singular_edges 
			+ count_exterior_edges == a1.number_of_finite_edges())
	

		assert(  count_exterior_vertices
		+ count_regular_vertices
		+ count_singular_vertices
		+ count_interior_vertices == a1.number_of_vertices())
	
	
	ncc = a1.number_of_solid_components()
	
	if (a1.get_mode()== Alpha_shape_3.Mode.REGULARIZED):
		#this relation might not be valid for any alpha_shape
		# if connected components are touching
		# through an edge or a vertex
		assert(count_regular_facets == 2*count_regular_vertices - 4*ncc)


if __name__ == '__main__':
	test_Alpha_shapes_3()




