#!/usr/bin/python
from sys import *
path.append("../../cgal_package")

from CGAL import *
from random import *

def test_constructors():
	"""
	>>> Polygon_2()
	Polygon_2()
	>>> p = Polygon_2()
	>>> Polygon_2(p)
	Polygon_2()
	>>> Polygon_2([Point_2(-1,0), Point_2(0,-1), Point_2(1,0), Point_2(0,1)])
	Polygon_2([Point_2(-1.0,0.0), Point_2(0.0,-1.0), Point_2(1.0,0.0), Point_2(0.0,1.0)])
	>>> Polygon_2([])
	Polygon_2()
	"""
	pass

def test_access():
	"""
	>>> p = Polygon_2([Point_2(-1.0,0.0), Point_2(0.0,-1.0), Point_2(1.0,0.0), Point_2(0.0,1.0)])
	>>> p[0]
	Point_2(-1.0,0.0)
	>>> p[3]
	Point_2(0.0,1.0)
	>>> p[-3]
	Point_2(0.0,-1.0)
	>>> p.edge(0)
	Segment_2(Point_2(-1.0,0.0),Point_2(0.0,-1.0))
	>>> p.edge(3)
	Segment_2(Point_2(0.0,1.0),Point_2(-1.0,0.0))
	>>> p.edge(4)
	Traceback (most recent call last):
	  File "<stdin>", line 1, in ?
	IndexError: Index out of range
	>>> p.edge(-1)
	Traceback (most recent call last):
	  File "<stdin>", line 1, in ?
	IndexError: Index out of range
	>>> for i, j in zip(p.vertices, p):
	...     assert i == j
	...
	>>> for i in p:
	...     print i
	...
	Point_2(-1.0,0.0)
	Point_2(0.0,-1.0)
	Point_2(1.0,0.0)
	Point_2(0.0,1.0)
	>>> p[0] = Point_2(-2,0)
	>>> p
	Polygon_2([Point_2(-2.0,0.0), Point_2(0.0,-1.0), Point_2(1.0,0.0), Point_2(0.0,1.0)])
	>>> p[-1] = Point_2(0,2)
	>>> p
	Polygon_2([Point_2(-2.0,0.0), Point_2(0.0,-1.0), Point_2(1.0,0.0), Point_2(0.0,2.0)])
	>>> n = 6
	>>> for v in p.vertices_circulator:
	...     print v
	...     n -= 1
	...     if n <= 0: break;
	...
	Point_2(-2.0,0.0)
	Point_2(0.0,-1.0)
	Point_2(1.0,0.0)
	Point_2(0.0,2.0)
	Point_2(-2.0,0.0)
	Point_2(0.0,-1.0)
	>>> n = 6
	>>> for e in p.edges_circulator:
	...     print e
	...     n -= 1
	...     if n <= 0: break;
	...
	Segment_2(Point_2(-2.0,0.0),Point_2(0.0,-1.0))
	Segment_2(Point_2(0.0,-1.0),Point_2(1.0,0.0))
	Segment_2(Point_2(1.0,0.0),Point_2(0.0,2.0))
	Segment_2(Point_2(0.0,2.0),Point_2(-2.0,0.0))
	Segment_2(Point_2(-2.0,0.0),Point_2(0.0,-1.0))
	Segment_2(Point_2(0.0,-1.0),Point_2(1.0,0.0))
	>>> for e in p.edges:
	...     print e
	...
	Segment_2(Point_2(-2.0,0.0),Point_2(0.0,-1.0))
	Segment_2(Point_2(0.0,-1.0),Point_2(1.0,0.0))
	Segment_2(Point_2(1.0,0.0),Point_2(0.0,2.0))
	Segment_2(Point_2(0.0,2.0),Point_2(-2.0,0.0))
	"""
	pass

def test_insertions():
	"""
	>>> p = Polygon_2()
	>>> p.push_back(Point_2(0,0))
	>>> p.push_back(Point_2(0,1))
	>>> p.push_back(Point_2(2,1))
	>>> p.push_back(Point_2(2,3))
	>>> p
	Polygon_2([Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p_original = Polygon_2(p)
	>>> p.insert(0, Point_2(-1, -1))
	>>> p
	Polygon_2([Point_2(-1.0,-1.0), Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p.insert(5, Point_2(-2, -2))
	>>> p
	Polygon_2([Point_2(-1.0,-1.0), Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0), Point_2(-2.0,-2.0)])
	>>> p.insert(3, Point_2(-3, -3))
	>>> p
	Polygon_2([Point_2(-1.0,-1.0), Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(-3.0,-3.0), Point_2(2.0,1.0), Point_2(2.0,3.0), Point_2(-2.0,-2.0)])
	>>> p.erase(0)
	>>> del p[2]
	>>> del p[len(p) - 1]
	>>> p == p_original
	True
	"""
	pass
	
def test_equality():
	"""
	>>> p1 = Polygon_2([Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p2 = Polygon_2([Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p1 == p2
	True
	>>> for i in xrange(10):
	...     p2.push_back(p2[0])
	...     del p2[0]
	...
	>>> p1
	Polygon_2([Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p2
	Polygon_2([Point_2(2.0,1.0), Point_2(2.0,3.0), Point_2(0.0,0.0), Point_2(0.0,1.0)])
	>>> p1 == p2
	True
	>>> p2[0] = Point_2(2.0,3.0)
	>>> p1 == p2
	False
	>>> p1 != p2
	True
	>>> del p2[0]
	>>> p1 != p2
	True
	>>> del p1[2]
	>>> p1 != p2
	False
	>>> p1 == p2
	True
	"""
	pass
	
def test_container():
	"""
	>>> p1 = Polygon_2([Point_2(0.0,0.0), Point_2(0.0,1.0), Point_2(2.0,1.0), Point_2(2.0,3.0)])
	>>> p1.is_empty()
	False
	>>> p1.clear()
	>>> p1.is_empty()
	True
	>>> p1
	Polygon_2()
	>>> p1.size()
	0
	>>> len(p1)
	0
	>>> p1.push_back(Point_2(0.0,0.0))
	>>> p1.size()
	1
	>>> len(p1)
	1
	"""
	pass
	
def test_misc():
	"""
	>>> p = Polygon_2([Point_2(-1.0,0.0), Point_2(0.0,-1.0), Point_2(1.0,0.0), Point_2(0.0,1.0)])
	>>> p.is_simple()
	True
	>>> p.is_convex()
	True
	>>> p.left_vertex_index()
	0
	>>> p.right_vertex_index()
	2
	>>> p.top_vertex_index()
	3
	>>> p.bottom_vertex_index()
	1
	>>> p.orientation()
	CGAL.Kernel.Sign.LARGER
	>>> p2 = Polygon_2()
	>>> for i in p: p2.insert(0, i)
	...
	>>> p2
	Polygon_2([Point_2(0.0,1.0), Point_2(1.0,0.0), Point_2(0.0,-1.0), Point_2(-1.0,0.0)])
	>>> p2.orientation()
	CGAL.Kernel.Sign.SMALLER
	>>> p.bounded_side(Point_2(0,0))
	CGAL.Kernel.Bounded_side.ON_BOUNDED_SIDE
	>>> p.bounded_side(Point_2(0.5, 0.5))
        CGAL.Kernel.Bounded_side.ON_BOUNDARY
	>>> p.bounded_side(Point_2(0,100))
	CGAL.Kernel.Bounded_side.ON_UNBOUNDED_SIDE
	>>> p.bbox()
	Bbox_2( xmin = -1.0,ymin = -1.0,xmax = 1.0,ymax = 1.0)
	>>> p.oriented_side(Point_2(0,0))
	CGAL.Kernel.Sign.LARGER
	>>> p.oriented_side(Point_2(0,100))
	CGAL.Kernel.Sign.SMALLER
	>>> p.area()
	2.0
	>>> p.is_counterclockwise_oriented()
	True
	>>> p.is_clockwise_oriented()
	False
	>>> p2.is_clockwise_oriented()
	True
	>>> p2.is_counterclockwise_oriented()
	False
	>>> p.is_collinear_oriented()
	False
	>>> p.has_on_positive_side(Point_2(0,0))
	True
	>>> p.has_on_positive_side(Point_2(0,10))
	False
	>>> p.has_on_negative_side(Point_2(0,10))
	True
	>>> p.has_on_negative_side(Point_2(0,0))
	False
	>>> p.has_on_boundary(Point_2(0,0))
	False
	>>> p.has_on_boundary(Point_2(1,0))
	True
	>>> p.has_on_boundary(Point_2(0.5,0.5))
	True
	>>> p.has_on_bounded_side(Point_2(0,0))
	True
	>>> p.has_on_bounded_side(Point_2(0,10))
	False
	>>> p.has_on_unbounded_side(Point_2(0,10))
	True
	>>> p.has_on_unbounded_side(Point_2(0,0))
	False
	>>> p.is_simple()
	True
	>>> p.is_convex()
	True
	>>> p.push_back(Point_2(0,0))
	>>> p.is_convex()
	False
	>>> p.is_simple()
	True
	>>> p.push_back(Point_2(2,-2))
	>>> p.is_simple()
	False
	>>> p.is_convex()
	False
	>>>
	"""
	pass

def _test():
    import doctest
    doctest.testmod()

#Let doctest work its magic
_test()
