#!/usr/bin/python
print "======================================================"
print "============     testing Polyhedron_3   =============="
print "======================================================\n"

from sys import *
path.append("../../cgal_package")

from CGAL.Polyhedron import *
from CGAL.Kernel import *

def Point_3_str(self):
    return "Point_3"+str((self.x(), self.y(), self.z()))
# now we turn it into a member function
Point_3.__str__ = Point_3_str

def test_Polyhedron():
	
	P = Polyhedron_3()
	hh = P.make_triangle()
	assert( P.is_triangle( hh))
	assert( P.is_triangle( Halfedge(hh)))
	assert( P.is_closed() == False)


#         # Next the conversions to Halfedge_iterator.
	hh = hh.opposite()
	P.fill_hole(hh)

  	assert( P.is_triangle( hh.opposite()) == False)

# 	# The first check that the polyhedron and its normalization works.
	P = Polyhedron_3()
# 	assert( 0 == halfedgeds_connected_components(P))
	h = P.make_triangle()
	assert( P.is_valid())
	assert( P.is_triangle( h))
	assert( P.is_tetrahedron( h) == False)
	assert( P.is_closed() == False)
	assert( h.vertex_degree() == 2)
	assert( h.vertex().vertex_degree() == 2)
	assert( h.is_bivalent())
	assert( h.vertex().is_bivalent())
	assert( P.is_pure_bivalent())
	assert( h.facet_degree() == 3)
	assert( h.facet().facet_degree() == 3)
	assert( h.is_triangle())
	assert( h.facet().is_triangle())
	assert( P.is_pure_triangle())
	P.normalize_border()
	assert( P.is_valid( False, 1))
	P.inside_out()
	assert( P.is_valid( False, 1))
	assert( P.is_triangle( h))
	P.normalize_border()
	assert( P.is_valid( False, 1))
	assert( P.is_triangle( h))
# 
	h = P.make_tetrahedron()
	assert( P.is_valid())
	assert( P.is_tetrahedron( h))
	assert( P.is_triangle( h) == False)
	assert( P.is_closed() == False) ## we have also a triangle in P
	assert( h.vertex_degree() == 3)
	assert( h.vertex().vertex_degree() == 3)
	assert( h.is_trivalent())
	assert( h.vertex().is_trivalent())
	assert( P.is_pure_bivalent() == False )
	assert( P.is_pure_trivalent() == False) ## there is a triangle too
	assert( h.facet_degree() == 3)
	assert( h.facet().facet_degree() == 3)
	assert( P.is_pure_triangle())

	P.normalize_border()
	assert( P.is_valid( False, 1))
	P.inside_out()
	assert( P.is_valid( False, 1))
	assert( P.is_tetrahedron( h))
	P.normalize_border()
	assert( P.is_valid( False, 1))
	assert( P.is_tetrahedron( h))
	P.make_hole(h)
	assert( P.is_tetrahedron( h) == False)
	P.fill_hole(h)
	assert( P.is_tetrahedron( h) )


# 	# Check the predefined point iterator
	try:
		P = Polyhedron_3()
		h = P.make_tetrahedron(Point_3( 1.0, 0.0, 0.0), Point_3( 0.0, 1.0, 0.0),Point_3( 0.0, 0.0, 1.0), Point_3( 0.0, 0.0, 0.0))
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		assert( P.is_pure_trivalent())
		points = P.points
		vertices = P.vertices
		for v in vertices:
			pt = points.next()
			assert(v.point() == pt)
	# 		print v.point(),"==" ,pt
		print "The predefined Point_iterator and Vertex_iterator are:\t\t OK"
	except:
		print "The predefined Point_iterator and Vertex_iterator are:\t\t NOT OK"
	


# 	# Check the predefined plane iterator
	try:	
		P = Polyhedron_3()
		P.make_tetrahedron()
		planes = P.planes
		facets = P.facets
	
		for p in planes:
			f = facets.next()
			assert( p == f.plane())
		print "The predefined Plane_iterator and Facet_iterator are:\t\t OK"
	except:
		print "The predefined Plane_iterator and Facet_iterator are:\t\t NOT OK"

# 	# Check border facet generation.
	try:
		P = Polyhedron_3()
		h = P.make_triangle()
		assert( P.is_valid())
		assert( P.is_triangle( h))
	
		g = P.add_vertex_and_facet_to_border(h.next().opposite(),h.opposite())
		
		assert( P.is_valid())
		assert( h.next().next().next() == h)
		assert( g.next().next().next() == g)
		assert( h.opposite().next() == g)
	
		gg = P.add_facet_to_border(h.next().next().opposite(),g.next().opposite())
		
		assert( P.is_valid())
		assert(  h.next().next().next() == h)
		assert(  g.next().next().next() == g)
		assert( gg.next().next().next() == gg)
		assert( h.opposite().next() == g)
		assert( gg.next().opposite() == h.next())
		print "Border facet generation is:\t\t\t\t\t OK"
	except:
		print "Border facet generation is:\t\t\t\t\t NOT OK"


#  # Check erasing of facets and connected components.

	try:
		
		P = Polyhedron_3()
		h = P.make_tetrahedron()
		g = P.make_tetrahedron()
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		assert( P.is_tetrahedron( g))
		P.erase_connected_component(h)
		assert( P.is_valid())
		assert( P.is_tetrahedron( g))
		assert( P.size_of_vertices()  == 4)
		assert( P.size_of_halfedges() == 12)
		assert( P.size_of_facets()    == 4)
		P.erase_connected_component( P.make_triangle())
		assert( P.is_valid())
		assert( P.is_tetrahedron( g))
		assert( P.size_of_vertices()  == 4)
		assert( P.size_of_halfedges() == 12)
		assert( P.size_of_facets()    == 4)
		P.erase_facet( g)
		assert( P.is_valid())
		assert( P.size_of_vertices()  == 4)
		assert( P.size_of_halfedges() == 12)
		assert( P.size_of_facets()    == 3)
		P.erase_facet( g.opposite())
		assert( P.is_valid())
		assert( P.size_of_vertices()  == 4)
		assert( P.size_of_halfedges() == 10)
		assert( P.size_of_facets()    == 2)
		P.clear()
		assert( P.is_valid())
		assert( P.size_of_vertices()  == 0)
		assert( P.size_of_halfedges() == 0)
		assert( P.size_of_facets()    == 0)
		print "Erasing of facets and connected components is:\t\t\t OK"
	except:
		print "Erasing of facets and connected components is:\t\t\t NOT OK"

# 	# Check bug-fix in join_vertex with respect to border edges.

	P = Polyhedron_3()
	h = P.make_triangle()
	P.split_vertex( h, h.next().opposite())
	P.join_vertex( h)
	assert( P.is_valid())



# 	# Check set_halfedge() for vertices and facets
	try:
		P = Polyhedron_3()
		h = P.make_tetrahedron()
		h.vertex().set_halfedge(h)
		assert( h.vertex().halfedge() == h)
		assert( h.vertex() == h.next().opposite().vertex())
		h.vertex().set_halfedge(h.next().opposite())
		assert( h.vertex().halfedge() == h.next().opposite())
		assert( P.is_valid())
	
		h.facet().set_halfedge(h)
		assert( h.facet().halfedge() == h)
		assert( h.facet() == h.next().facet())
		h.facet().set_halfedge(h.next())
		assert( h.facet().halfedge() == h.next())
		assert( P.is_valid())
		print "set_halfedge() for vertices and facets  is:\t\t\t OK"
	except:
		print "set_halfedge() for vertices and facets  is:\t\t\t NOT OK"
		

#     
#         # Check invariants of Euler operations.
	try:
		P = Polyhedron_3()
		h = P.make_tetrahedron()
		assert( P.is_tetrahedron( h))
		g = P.split_vertex( h, h.next().opposite())
		assert( g.facet_degree() == 4)
		assert( g.facet().facet_degree() == 4)
		assert( g.is_quad())
		assert( g.facet().is_quad())
		assert( g == h.next().opposite())
		assert( P.is_valid())
		i = P.join_vertex( g)
		assert( i == h)
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		
		g = P.split_vertex( h, h.next().opposite())
		assert( g == h.next().opposite())
		assert( P.is_valid())
		# Create a diagonal in the quadrangle.
		i = P.split_facet( g, g.next().next())
		assert( i == g.next())
		assert( P.is_valid())
		# Make diagonal flip four times until it reaches its original position
		i = P.flip_edge( i)
		assert( P.is_valid())
		i = P.flip_edge( i)
		assert( P.is_valid())
		i = P.flip_edge( i)
		assert( P.is_valid())
		i = P.flip_edge( i)
		assert( P.is_valid())
		# remove diagonal
		j = P.join_facet( i)
		assert( j == g)
		assert( P.is_valid())
		# collapse quadrilateral to triangle
		j = P.join_vertex( g)
		assert( j == h)
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		g = P.make_tetrahedron()
		gg = g.opposite().next()
		
		assert( P.size_of_vertices()  == 8)
		assert( P.size_of_halfedges() == 24)
		assert( P.size_of_facets()    == 8)
		assert( P.is_valid())
		i = P.join_loop( h, g)
		assert( i == h)
		assert( P.size_of_vertices()  == 5)
		assert( P.size_of_halfedges() == 18)
		assert( P.size_of_facets()    == 6)
		# The unchanged facets.
		assert( h.opposite().facet().halfedge().facet() == h.opposite().facet())
		assert( h.opposite().next_on_vertex().facet().halfedge().facet() == h.opposite().next_on_vertex().facet())
		assert( h.opposite().next_on_vertex().next_on_vertex().facet().halfedge().facet() == h.opposite().next_on_vertex().next_on_vertex().facet())
		# The changed facets.
		assert( h.facet().halfedge().facet() == h.facet())
		assert( h.next_on_vertex().facet().halfedge().facet() == h.next_on_vertex().facet())
		assert( h.next_on_vertex().next_on_vertex().facet().halfedge().facet() == h.next_on_vertex().next_on_vertex().facet())
		assert( P.is_valid())
		i = h.next().opposite().next()
		j = i.next().opposite().next()
		i = P.split_loop( h, i, j)
		assert( i.opposite().next() == gg)
		assert( P.size_of_vertices()  == 8)
		assert( P.size_of_halfedges() == 24)
		assert( P.size_of_facets()    == 8)
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		assert( P.is_tetrahedron( i))
		
		# create and erase center_vertex
		i = P.create_center_vertex( h)
		assert( i == h.next())
		assert( P.is_valid())
		j = P.erase_center_vertex( i)
		assert( j == h)
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		print "Invariants of Euler operations is:\t\t\t\t OK"
	except:
		print "Invariants of Euler operations is:\t\t\t\t NOT OK"

#         # The first check that the polyhedron and its normalization works.
	try:
		P = Polyhedron_3()
		h = P.make_triangle()
		assert( P.is_valid())
		assert( P.is_triangle( h))
		assert( P.is_tetrahedron( h) == False)
		P.normalize_border()
		assert( P.is_valid( False, 1))
		P.inside_out()
		assert( P.is_valid( False, 1))
		assert( P.is_triangle( h))
		P.normalize_border()
		assert( P.is_valid( False, 1))
		assert( P.is_triangle( h))
	
		h = P.make_tetrahedron()
		assert( P.is_valid())
		assert( P.is_tetrahedron( h))
		assert( P.is_triangle( h) == False)
		P.normalize_border()
		assert( P.is_valid( False, 1))
		P.inside_out()
		assert( P.is_valid( False, 1))
		assert( P.is_tetrahedron( h))
		P.normalize_border()
		assert( P.is_valid( False, 1))
		assert( P.is_tetrahedron( h))
		P.make_hole(h)
		assert( P.is_tetrahedron( h) == False)
		P.fill_hole(h)
		assert( P.is_tetrahedron( h))
		print "Checking that the polyhedron and its normalization works is:\t OK"
	except:
		print "Checking that the polyhedron and its normalization works is:\t NOT OK"


if __name__ == '__main__':
	test_Polyhedron()