#!/usr/bin/python
print "======================================================"
print "============     testing Delaunay_triangulation_3   ======"
print "======================================================\n"

from sys import *
path.append("../../cgal_package")

from CGAL.Triangulations_3 import *
from CGAL.Kernel import *

def  Point_3_str(self):
    return " Point_3"+str((self.x(), self.y(), self.z()))
# now we turn it into a member function
Point_3.__str__ =  Point_3_str


def test_conflicts(T3_13,q):
	for p in q:
		if (T3_13.dimension() < 2):
			T3_13.insert(p)
		else:
			# First locate the point.
			l=[]
			c = T3_13.locate(p, l); lt =l[0]; li=l[1]; lj=l[2]
			if (lt == Triangulation_3.Locate_type.VERTEX): 
				continue
			if (lt == Triangulation_3.Locate_type.OUTSIDE_AFFINE_HULL):
				T3_13.insert_outside_affine_hull(p)
				continue
			# Get the stuff in conflicts.
			V = T3_13.vertices_in_conflict(p, c)
			Tuple = T3_13.find_conflicts(p, c)
			F = Tuple[0]
			C = Tuple[1]
			if (T3_13.dimension() == 3):
				assert(len(F) == 2*len(V) - 4) # Euler relation.
			if (T3_13.dimension() == 2):
				assert(len(F) == len(V))				  
			T3_13.insert_in_hole(p, C,F[0].cell(), F[0].vertex_index());			
				  
				  
def test_delaunay_triangulation_3():

	l1 = [  Point_3(0,0,0), Point_3(0,0,1), Point_3(0,0,2), Point_3(0,0,3), Point_3(0,0,4)]
	l2= [  Point_3(0,0,0), Point_3(0,0,4), Point_3(0,0,1), Point_3(0,0,2), Point_3(0,0,3)] 
	l3 =[ Point_3(0,0,1), Point_3(0,0,2), Point_3(0,0,3), Point_3(0,0,0), Point_3(0,0,4)] 
	#  Point_3s for T2_0 :
	p1= Point_3(5,5,0)  
	p2= Point_3(4,4,0) 
	p3= Point_3(6,6,0)  # 1- dimensional until this point
	p4= Point_3(5,3,0)  # 2-dimensional
	p5= Point_3(5,7,0)  
	p6= Point_3(5,4,0)  
	p7= Point_3(5,6,0)  
	p8= Point_3(0,0,0)  
	p9= Point_3(5,5,0)  
	
	#  Point_3s for T3_1 :
	q = [
		Point_3(0,0,0),  Point_3(4,4,0),  Point_3(0,4,0),  Point_3(4,0,0),
		Point_3(1,3,1),  Point_3(3,1,1),  Point_3(3,3,1),  Point_3(1,1,1),  Point_3(2,2,2),
		Point_3(1,3,3),  Point_3(3,1,3),  Point_3(3,3,3),  Point_3(1,1,3), 
		Point_3(0,0,4),  Point_3(4,4,4),  Point_3(0,4,4),  Point_3(4,0,4),
		Point_3(1,3,5),  Point_3(3,1,5),  Point_3(3,3,5),  Point_3(1,1,5),  Point_3(2,2,6)
		] 
	
	#  Point_3s for T3_2 :
	
	lp =[] 
	
	for a in range(10):
		for b in range(10):
			for d in range(10):
				lp.append( Point_3(a*b-d*a + (a-b)*10 +a ,a-b+d +5*b,a*a-d*d+b)) 
	
	#  Point_3s for T3_2 :
	
	lp2 =[]
	for a in range(4):
		for b in range(4):
			for d in range(4):
				lp2.append( Point_3((a*b-d*a)*10 +a ,(a-b+d +5*b)*100,a*a-d*d-b)) 
	
	#########################################################################

	print "Constructor "
	T0 = Delaunay_triangulation_3()  
	assert(T0.dimension() == -1) 
	assert(T0.number_of_vertices() == 0) 
	assert(T0.is_valid()) 
	
	
	print "Constructor1 "  
	p10 =Point_3(0,0,0) 
	v0 = T0.insert(p10) 
	assert(T0.dimension() == 0) 
	assert(T0.number_of_vertices() == 1) 
	assert(T0.is_valid()) 
	
	print "Constructor2 "  
	
	p11 = Point_3(100,100,0) 
	v0=T0.insert(p11) 
	assert(T0.dimension() == 1) 
	assert(T0.number_of_vertices() == 2) 
	assert(T0.is_valid()) 
	
	print "Constructor3 "  
	
	p12 = Point_3(100,-100,0) 
	v0=T0.insert(p12) 
	assert(T0.dimension() == 2) 
	assert(T0.number_of_vertices() == 3) 
	assert(T0.is_valid()) 
	
	print "Constructor4 "  
	
	p13 = Point_3(50,0,100) 
	v0 = T0.insert(p13) 
	assert(T0.dimension() == 3) 
	assert(T0.number_of_vertices() == 4) 
	assert(T0.is_valid()) 
	
	print "Constructor5 "  
	
	p14 = Point_3(50,0,100) 
	v0=T0.insert(p14) 
	assert(T0.dimension() == 3) 
	assert(T0.number_of_vertices() == 4) 
	assert(T0.is_valid()) 
	
	# copy constructor
	T1 = Delaunay_triangulation_3(T0) 
	assert(T1.dimension() == 3) 
	assert(T1.number_of_vertices() == 4) 
	assert(T1.is_valid()) 
	
	T1.clear() 
	assert(T1.dimension() == -1) 
	assert(T1.number_of_vertices() == 0) 
	assert(T1.is_valid()) 
	
	T1.swap(T0) 
	assert(T1.dimension() == 3) 
	assert(T1.number_of_vertices() == 4) 
	assert(T1.is_valid()) 
	assert(T0.dimension() == -1) 
	assert(T0.number_of_vertices() == 0) 
	assert(T0.is_valid()) 
	T0.swap(T1) 
	
	assert(T0.dimension() == 3) 
	assert(T0.number_of_vertices() == 4) 
	assert(T0.is_valid()) 
	assert(T1.dimension() == -1) 
	
		# Building some psychotics triangulations :
	print "Constructor6 "  
	# triangulation 1-dimensional : vertical line.
	T1_0 = Delaunay_triangulation_3()
	n = T1_0.insert(l1) 
	assert(n==5) 
	assert(T1_0.dimension()==1) 
	assert(T1_0.number_of_vertices()==n) 
	assert(T1_0.is_valid()) 
	print "Constructor7 "  
	T1_1 = Delaunay_triangulation_3()
	n = T1_1.insert(l2) 
	assert(n==5) 
	assert(T1_1.dimension()==1) 
	assert(T1_1.number_of_vertices()==n) 
	assert(T1_1.is_valid()) 
	print "Constructor8 "  
	T1_2 = Delaunay_triangulation_3()
	n = T1_2.insert(l3) 
	assert(n==5) 
	assert(T1_2.dimension()==1) 
	assert(T1_2.number_of_vertices()==n) 
	assert(T1_2.is_valid()) 
	
	print "Constructor9 "  
	# 2-dimensional triangulations 
	
	T2_0 = Delaunay_triangulation_3()
	v0=T2_0.insert(p1) 
	
	v0=T2_0.insert(p2) 
	
	v0=T2_0.insert(p3) 
	
	assert(T2_0.is_valid()) 
	assert(T2_0.dimension()==1) 
	assert(T2_0.number_of_vertices()==3) 
	
	
	v0=T2_0.insert(p4) 
	assert(T2_0.is_valid()) 
	assert(T2_0.dimension()==2) 
	assert(T2_0.number_of_vertices()==4) 
	
	
	v0=T2_0.insert(p5) 
	v0=T2_0.insert(p6) 
	v0=T2_0.insert(p7) 
	v0=T2_0.insert(p8) 
	v0=T2_0.insert(p9) 
	
	assert(T2_0.is_valid()) 
	assert(T2_0.dimension()==2) 
	assert(T2_0.number_of_vertices()==8) 
	
	print "Constructor10 "  
  # test grid insert
  	T2_1 = Delaunay_triangulation_3()
 
	px=1; py=1 
	qx=-1; qy=2
	for m in range(20):
		for n in range(20):
			p = Point_3(m*px+int(n*qx), m*py+int(n*qy), 1)
			T2_1.insert(p) 
	assert( T2_1.dimension()==2 ) 
	assert( T2_1.is_valid() ) 
	
	print "Constructor11 "  
	# 3-dimensional triangulations
	# This is a simple grid :
	T3_0 = Delaunay_triangulation_3()
	r ={} 
	for z in range(5):
		for y in range(5):
			for x in range(5): 
				r[x+5*y+25*z] =  Point_3(x,y,z) 
				T3_0.insert(r[x+5*y+25*z]) 
				
	assert(T3_0.is_valid()) 
	assert(T3_0.number_of_vertices()==125) 
	assert(T3_0.dimension()==3) 

	print "Constructor12 "  
	T3_1 = Delaunay_triangulation_3()
	for i in range(22):
		T3_1.insert(q[i]) 
	assert(T3_1.is_valid()) 
	assert(T3_1.number_of_vertices()==22) 
	assert(T3_1.dimension()==3) 
# 
#   Testing find_conflicts(), vertices_in_conflict(), insert_in_hole()
#   # FIXME : Note that we do not test the version of find_conflicts()
#   #         which returns the internal facets too...
	print "Testing find_conflicts/vertices_in_conflict/insert_in_hole"
				
	T3_13 = Delaunay_triangulation_3()
	
	test_conflicts(T3_13,q)
	
	assert(T3_13.is_valid()) 
	assert(T3_13.number_of_vertices()==22) 
	assert(T3_13.dimension()==3)

	print "Testing move_point()"
		
	T = Delaunay_triangulation_3()
	L ={}
	for i,p in enumerate(q):
		L[i] = T.insert(p) 
	assert(T.is_valid()) 
	assert(T.number_of_vertices()==22) 
	assert(T.dimension()==3) 
	
	for i in range(5):
		T.move_point(L[i],q[i*2])
	
	
	print "Test is_Gabriel "  
	q0  = Point_3(0.,0.,0.) 
	q1 = Point_3 (2.,0.,0.) 
	q2 = Point_3(0.,2.,0.) 
	q3 = Point_3(0.,0.,2.) 
	T4 = Delaunay_triangulation_3()
	
	v0 = T4.insert(q0) 
	v1 = T4.insert(q1) 
	v2 = T4.insert(q2) 
	v3 = T4.insert(q3)
	L = [] 
	c = Cell()
	assert(T4.is_facet(v0,v1,v2,c,L));j=L[0];k=L[1];l=L[2] 
	i = 6 - (j+k+l) 
	f = Facet(c,i) 
	assert(T4.is_Gabriel(c,i)) 
	assert(T4.is_Gabriel(f))
	L=[] 
	assert(T4.is_facet(v1,v2,v3,c,L));j=L[0];k=L[1];l=L[2] 
	i = 6 - (j+k+l) 
	assert(T4.is_Gabriel(c,i) == False)
	L=[] 
	e = Edge()
	assert(T4.is_edge(v0,v1,e))
	assert(T4.is_Gabriel(e.cell(),e.index(0),e.index(1))) 
	assert(T4.is_Gabriel(e)) 
	e = Edge()
	assert(T4.is_edge(v2,v3,e))
	assert(T4.is_Gabriel(e.cell(),e.index(0),e.index(1))) 
#     
	print"Test destructors and quit " 
	
	T0.clear() 
	assert(T0.is_valid()) 
	T3_0.clear() 
	assert(T3_0.is_valid()) 
	assert(T3_0.dimension()==-1) 
	assert(T3_0.number_of_vertices()==0) 


if __name__ == '__main__':
	test_delaunay_triangulation_3()