#!/usr/bin/python
print "======================================================"
print "============     testing Triangulation_3   =============="
print "======================================================\n"

from sys import *
path.append("../../cgal_package")

from CGAL.Triangulations_3 import *
from CGAL.Kernel import *

def Point_3_str(self):
    return "Point_3"+str((self.x(), self.y(), self.z()))
# now we turn it into a member function
Point_3.__str__ = Point_3_str

def test_triangulation_3():

	l1 = [ Point_3(0,0,0),Point_3(0,0,1),Point_3(0,0,2),Point_3(0,0,3),Point_3(0,0,4)]
	l2 = [ Point_3(0,0,0),Point_3(0,0,4),Point_3(0,0,1),Point_3(0,0,2),Point_3(0,0,3)]
	
	l3 = [ Point_3(0,0,1),Point_3(0,0,2),Point_3(0,0,3),Point_3(0,0,0),Point_3(0,0,4)]

	# Points for T2_0 :
	p1=Point_3(5,5,0) 
	p2=Point_3(4,4,0)
	p3=Point_3(6,6,0) # 1- dimensional until this point
	p4=Point_3(5,3,0) # 2-dimensional
	p5=Point_3(5,7,0) 
	p6=Point_3(5,4,0) 
	p7=Point_3(5,6,0) 
	p8=Point_3(0,0,0) 
	p9=Point_3(5,5,0) 
  
  # Points for T3_1 :
	q = [
		Point_3(0,0,0), Point_3(4,4,0), Point_3(0,4,0), Point_3(4,0,0),
		Point_3(1,3,1), Point_3(3,1,1), Point_3(3,3,1), Point_3(1,1,1), Point_3(2,2,2),
		Point_3(1,3,3), Point_3(3,1,3), Point_3(3,3,3), Point_3(1,1,3), 
		Point_3(0,0,4), Point_3(4,4,4), Point_3(0,4,4), Point_3(4,0,4),
		Point_3(1,3,5), Point_3(3,1,5), Point_3(3,3,5), Point_3(1,1,5), Point_3(2,2,6)
		];
	
	print"Constructor "
	
	# Beginning with an empty triangulation and adding points until reaching
	# 3-dimensional triangulation.
	T0 = Triangulation_3() 
	assert(T0.dimension() == -1)
	assert(T0.number_of_vertices() == 0)
	assert(T0.is_valid())
	
	print"Constructor1 "
	p10 = Point_3(0,0,0)
	v0 = T0.insert(p10)
	assert(T0.dimension() == 0)
	assert(T0.number_of_vertices() == 1)
	assert(T0.is_valid())
	
	print"Constructor2 " 
	
	p11 = Point_3(100,100,0)
	v0 = T0.insert(p11)
	assert(T0.dimension() == 1)
	assert(T0.number_of_vertices() == 2)
	assert(T0.is_valid())

	print"Constructor3 " 
	
	p13 = Point_3(100,-100,0)
	v0 = T0.insert(p13)
	assert(T0.dimension() == 2)
	assert(T0.number_of_vertices() == 3)
	assert(T0.is_valid())
	
	print"Constructor4 " 
	
	p13 = Point_3(50,0,100)
	v0 = T0.insert(p13)
	assert(T0.dimension() == 3)
	assert(T0.number_of_vertices() == 4)
	assert(T0.is_valid())
	
	print"Constructor5 " 
	
	p14 = Point_3(50,0,100)
	v0 = T0.insert(p13)
	assert(T0.dimension() == 3)
	assert(T0.number_of_vertices() == 4)
	assert(T0.is_valid())


	# copy constructor
	T1 = Triangulation_3(T0)
	assert(T1.dimension() == 3)
	assert(T1.number_of_vertices() == 4)
	assert(T1.is_valid())
	
	print "Testing operator==" 
	assert(T0 == T0)
	assert(T0 == T1)
	assert(T1 == T0)
	assert(T1 == T1)
	
	T1.clear()
	assert(T1.dimension() == -1)
	assert(T1.number_of_vertices() == 0)
	assert(T1.is_valid())
	# Assignment # be carful with assignement in python, it 's not the same thing as in C++
	# in this example T1 will be the reference of T0
	T1 = T0  
	assert(T1.dimension() == 3)
	assert(T1.number_of_vertices() == 4)
	assert(T1.is_valid())
#  	T1.clear()
	
	T1 = Triangulation_3()
	try:
		T1.swap(T0)   
		assert(T1.dimension() == 3)
		assert(T1.number_of_vertices() == 4)
		assert(T1.is_valid())
		assert(T0.dimension() == -1)
		assert(T0.number_of_vertices() == 0)
		assert(T0.is_valid())
		T0.swap(T1)
		print "testing swap is: \t OK"
	except:
		print "testing swap is: \t NOT OK"
	
	assert(T0.dimension() == 3)
	assert(T0.number_of_vertices() == 4)
	assert(T0.is_valid())
	assert(T1.dimension() == -1)

	print "Constructor6" 
	T1_0 = Triangulation_3()
	n = T1_0.insert(l1)
	assert(n==5)
	assert(T1_0.dimension()==1)
	assert(T1_0.number_of_vertices()==n)
	assert(T1_0.is_valid())

	print"Constructor7 "
	T1_1 = Triangulation_3()
	n = T1_1.insert(l2)
	assert(n==5)
	assert(T1_1.dimension()==1)
	assert(T1_1.number_of_vertices()==n)
	assert(T1_1.is_valid())
	
	print"Constructor8 "
	T1_2 = Triangulation_3()
	n = T1_2.insert(l3)
	assert(n==5)
	assert(T1_2.dimension()==1)
	assert(T1_2.number_of_vertices()==n)

	print"Constructor9 "
	# 2-dimensional triangulations 
	
	T2_0 = Triangulation_3()
	v0=T2_0.insert(p1);
	
	v0=T2_0.insert(p2);
	
	v0=T2_0.insert(p3);
	
	assert(T2_0.is_valid());
	assert(T2_0.dimension()==1);
	assert(T2_0.number_of_vertices()==3);
	
	
	v0=T2_0.insert(p4);
	assert(T2_0.is_valid());
	assert(T2_0.dimension()==2);
	assert(T2_0.number_of_vertices()==4);
	
	
	v0=T2_0.insert(p5);
	v0=T2_0.insert(p6);
	v0=T2_0.insert(p7);
	v0=T2_0.insert(p8);
	v0=T2_0.insert(p9);
	
	assert(T2_0.is_valid());
	assert(T2_0.dimension()==2);
	assert(T2_0.number_of_vertices()==8);

	print"Constructor12 "
	T3_1 =Triangulation_3();
	
	T3_1.insert(q);
	assert(T3_1.is_valid());
	assert(T3_1.number_of_vertices()==22);
	assert(T3_1.dimension()==3);


	print "Testing insertions   "
	
	li,lj,i1,i2 = 0,0,0,0
	Ti = Triangulation_3(T0);
	p20 = Point_3(50,0,50);
	v0=Ti.insert_in_cell(p20,Ti.locate(Point_3(50,0,50)));
	assert(Ti.is_valid());
	assert(Ti.number_of_vertices() == 5);
	Ti=Triangulation_3(T0);
	p120 = Point_3(50,0,0);
	v0=Ti.insert_in_facet(p120,Ti.locate(Point_3(50,0,1)),3);
	assert(Ti.is_valid());
	assert(Ti.number_of_vertices() == 5);
	Ti=Triangulation_3(T0);
	p21 = Point_3(50,0,0);
	v0=Ti.insert_in_facet(p21,Facet(Ti.locate(Point_3(50,0,1)),3));
	assert(Ti.is_valid());
	assert(Ti.number_of_vertices() == 5);
	Ti=Triangulation_3(T0);
	l =[]
	c= Ti.locate(Point_3(0,0,0),l);lt = l[0];li = l[1];lj = l[2]
	assert(lt==Triangulation_3.Locate_type.VERTEX);
	i1=li;
	print l

# 	l=[]
	c= Ti.locate(Point_3(100,100,0),l);lt = l[0];li = l[1];lj = l[2]
	print l
	assert(lt==Triangulation_3.Locate_type.VERTEX);
	i2=li;
	
	p22 = Point_3(50,50,0);
	v0=Ti.insert_in_edge(p22,Ti.locate(Point_3(50,40,1)),i1,i2);
	assert(Ti.is_valid());
	assert(Ti.number_of_vertices() == 5);

	Ti=Triangulation_3(T0);
# 	l=[]
	c= Ti.locate(Point_3(0,0,0),l);lt = l[0];li = l[1];lj = l[2]
	assert(lt==Triangulation_3.Locate_type.VERTEX);
	i1=li;
	l=[]
	c= Ti.locate(Point_3(100,100,0),l);lt = l[0];li = l[1];lj = l[2]
	assert(lt==Triangulation_3.Locate_type.VERTEX);
	i2=li;
	p23 = Point_3(50,50,0);
	v0=Ti.insert_in_edge(p23,Edge(Ti.locate(Point_3(50,50,0)),i1,i2));
	assert(Ti.is_valid());
	assert(Ti.number_of_vertices() == 5);
	
	Ti = Triangulation_3(T0);	
	assert(T0.dimension() == 3);
	assert(T0.number_of_vertices() == 4);
	assert(T0.is_valid());
		
	l =[]
	c= Ti.locate(Point_3(50,50,50),l);lt = l[0];li = l[1];lj = l[2]		
	p24 = Point_3(50,50,50);
	v0= Ti.insert_outside_convex_hull(p24,c);
	assert(Ti.is_valid());	
	assert(Ti.number_of_vertices() == 5);
	
	T3_3=Triangulation_3(T1_0);
	p25 = Point_3(2,0,0);
	v0=T3_3.insert_outside_affine_hull(p25);
	assert(T3_3.is_valid());
	assert(T3_3.dimension()==2);
	l=[]
	c= T3_3.locate(Point_3(4,0,0),l);lt = l[0];li = l[1];lj = l[2]
	p26 = Point_3(4,0,0);
	v0=T3_3.insert_outside_convex_hull(p26,c);
	assert(T3_3.is_valid());
	assert(T3_3.dimension()==2);
	p27 = Point_3(0,5,0);
	v0=T3_3.insert_outside_affine_hull(p27);
	assert(T3_3.is_valid());
	assert(T3_3.dimension()==3);

#=============================
	print"Boolean and query functions "
	c=T0.infinite_cell();
	assert(T0.is_infinite(c));
	ind = c.index(T0.infinite_vertex());
		

	for i in range(4):
		if (i!=ind): 
			assert(T0.is_infinite(c,i));
			f=Facet(c,i);
			assert(T0.is_infinite(f));
		
	for i in range(4): 
		for j in range(4):
			if ((i!=j) and ((i==ind) or (j==ind))): 
				assert(T0.is_infinite(c,i,j));
				assert(T0.is_infinite(Edge(c,i,j)));
		
	v0=T0.infinite_vertex();
	assert(T0.is_infinite(v0));
	
	Finite_vertices = T3_1.vertices ;
	w = Vertex();
	for vit in Finite_vertices:
		assert(T3_1.is_vertex(vit.point(), w));
	
		# geometric functions
	print "Geometric functions "
	l = [] 
	c = T0.locate(Point_3(50,0,1),l);lt = l[0];li = l[1];lj = l[2]
	tr1 = T0.tetrahedron(c);
	l = []
	c = T0.locate(Point_3(10,0,1),l);lt = l[0];li = l[1];lj = l[2]
	tr2 = T0.tetrahedron(c);
	assert(tr1 == tr2);
	l =[]
	c= T0.locate(Point_3(50,0,5),l);lt = l[0];li = l[1];lj = l[2]
	tri1=T0.triangle(c,1);
	l =[]
	c= T0.locate(Point_3(10,0,1),l);lt = l[0];li = l[1];lj = l[2]
	tri2=T0.triangle(Facet(c,1));
	assert(tri1==tri2);
	l =[]
	c= T0.locate(Point_3(10,0,1),l);lt = l[0];li = l[1];lj = l[2]
	s1 = T0.segment(c,0,1);
	l =[]
	c= T0.locate(Point_3(50,0,5),l);lt = l[0];li = l[1];lj = l[2]
	s2 = T0.segment(Edge(c,0,1));
	assert(s1==s2);


if __name__ == '__main__':
	test_triangulation_3()